package speiger.src.coreengine.rendering.gui.renderer;

import java.util.List;
import java.util.function.Consumer;

import speiger.src.collections.ints.collections.IntStack;
import speiger.src.collections.ints.lists.IntArrayList;
import speiger.src.collections.objects.lists.ObjectArrayList;
import speiger.src.collections.utils.Stack;
import speiger.src.coreengine.math.misc.Facing;
import speiger.src.coreengine.math.vector.matrix.Matrix4f;
import speiger.src.coreengine.math.vector.matrix.Matrix4fStack;
import speiger.src.coreengine.math.vector.quaternion.Quaternion;
import speiger.src.coreengine.rendering.guiOld.helper.box.IGuiBox;
import speiger.src.coreengine.rendering.models.DrawCall;
import speiger.src.coreengine.rendering.tesselation.GLCall;
import speiger.src.coreengine.rendering.tesselation.buffer.VertexBuilder;

public class SimpleUIRenderer implements IUIRenderer, AutoCloseable {
	TexturedRect cachedRect = new TexturedRect();
	Matrix4fStack transform = new Matrix4fStack();
	List<Matrix4f> transformCache = new ObjectArrayList<>();
	IntStack indexCache = new IntArrayList();
	boolean matrixChanged = false;
	
	List<GLCall> drawCalls = new ObjectArrayList<>();
	Stack<GLCall> cache = new ObjectArrayList<>();
	
	VertexBuilder builder = new VertexBuilder(Short.MAX_VALUE);
	
	@Override
	public void close() throws Exception {
		builder.close();
	}
	
	@Override
	public boolean isInScissors(IGuiBox box) {
		return false;
	}
	
	@Override
	public void pushScissors(IGuiBox box) {
		
	}
	
	@Override
	public void popScissors() {
		
	}
	
	@Override
	public TexturedRect getCachedRect() { return cachedRect; }
	
	@Override
	public void flush() {
		
	}
	
	@Override
	public void pushTransform() {
		transform.push();
		
	}
	
	@Override
	public void popTransform() {
		transform.pop();
		if(transform.size() > indexCache.size()) {
			indexCache.pop();
		}
	}
	
	@Override
	public void translate(float z) {
		transform.translate(0, 0, z);
		matrixChanged = true;
	}
	
	@Override
	public void translate(float x, float y) {
		transform.translate(x, y);
		matrixChanged = true;
	}
	
	@Override
	public void translate(float x, float y, float z) {
		transform.translate(x, y, z);
		matrixChanged = true;
	}
	
	@Override
	public void scale(float scale) {
		transform.scale(scale);
		matrixChanged = true;
	}
	
	@Override
	public void scale(float x, float y) {
		transform.scale(x, y, 1F);
		matrixChanged = true;
	}
	
	@Override
	public void rotate(Quaternion rotation) {
		transform.rotate(rotation);
		matrixChanged = true;
	}
	
	@Override
	public void drawCustom(Consumer<Matrix4f> matrix) {
		
	}
	
	@Override
	public void drawCustom(DrawCall drawcall) {
		
	}
	
	@Override
	public void drawLine(float minX, float minY, float maxX, float maxY, float zLevel, int color) {
		
	}
	
	@Override
	public void drawFrame(float minX, float minY, float maxX, float maxY, float zLevel, int color) {
		
	}
	
	@Override
	public void drawRect(float minX, float minY, float maxX, float maxY, float zLevel, int color) {
		
	}
	
	@Override
	public void drawGradientRect(float minX, float minY, float maxX, float maxY, float zLevel, int startColor, int endColor, Facing facing) {
		
	}
	
	@Override
	public void drawTexturedRect(float minX, float minY, float maxX, float maxY, float zLevel, int texture, float minU, float minV, float maxU, float maxV, int color) {
		
	}
	
}
